import jwt
import time
import random
import logging

DEFAULT_ALGORITHM = 'RS512'
TIMESTAMP_TOLERANCE = 20 # Seconds

logger = logging.getLogger(__name__)


def sign(username, private_key, generate_nonce=None, iat=None, algorithm=DEFAULT_ALGORITHM):
    """Create a signed JWT using the given username and RSA private key"""

    iat = iat if iat else time.time()
    if not generate_nonce:
        generate_nonce = lambda username, iat: random.random()

    token_data = {
        'username': username,
        'time': iat,
        'nonce': generate_nonce(username, iat),
    }

    token = jwt.encode(token_data, private_key, algorithm=algorithm)
    return token


def get_claimed_username(token):
    unverified_data = jwt.decode(token, verify=False)
    if not unverified_data.has_key('username'):
        return None
    return unverified_data['username']


def verify(token, public_key, validate_nonce=None, algorithms=[DEFAULT_ALGORITHM]):
    """Verify the validity of the given JWT using a public key"""
    try:
        token_data = jwt.decode(token, public_key, algorithms=algorithms)
    except jwt.InvalidTokenError as e:
        logger.info('JWT failed verification', exc_info=e)
        return False

    claimed_username = token_data.get('username')
    claimed_time = token_data.get('time', 0)
    claimed_nonce = token_data.get('nonce')

    # Ensure time is within acceptable bounds
    current_time = time.time()
    min_time, max_time = (current_time - TIMESTAMP_TOLERANCE, current_time + TIMESTAMP_TOLERANCE)
    if claimed_time < min_time or claimed_time > max_time:
        logger.info('Claimed time is outside of allowable tolerances')
        return False

    # Ensure nonce is unique
    if validate_nonce:
        if not validate_nonce(claimed_username, claimed_time, claimed_nonce):
            logger.info('Claimed nonce failed to validate')
            return False
    else:
        logger.warning('validate_nonce function was not supplied!')

    # If we've gotten this far, the token is valid
    return token_data



from django.core.management.base import NoArgsCommand
from asymmetric_jwt_auth import generate_key_pair


class Command(NoArgsCommand):
    help = "Generate a public / private RSA key pair"

    def handle_noargs(self, **options):
        pem_private, pem_public = generate_key_pair()
        print(pem_private)
        print(pem_public)



import unittest
import time
import os.path
from asymmetric_jwt_auth import create_auth_header, generate_key_pair
from asymmetric_jwt_auth.token import verify

BASE = os.path.dirname(os.path.abspath(__file__))

KEY1_PRIVATE = os.path.join(BASE, 'dummy')
KEY1_PUBLIC = os.path.join(BASE, 'dummy.pub')

KEY2_PRIVATE = os.path.join(BASE, 'dummy_encrypted')
KEY2_PUBLIC = os.path.join(BASE, 'dummy_encrypted.pub')
KEY2_PASSWORD = b'password'


class HTTPHeaderTest(unittest.TestCase):
    def test_generate_from_key_string(self):
        private1, public1 = generate_key_pair()
        private2, public2 = generate_key_pair()
        header = create_auth_header('foo', key=private1)
        self.assertTrue(header.startswith('JWT '))
        token = header.split(' ', 1)[1]
        self.assertFalse(verify(token, public2))
        self.assertTrue(verify(token, public1))

    def test_generate_from_key_file(self):
        header = create_auth_header('foo', key_file=KEY1_PRIVATE)
        self.assertTrue(header.startswith('JWT '))
        token = header.split(' ', 1)[1]
        with open(KEY1_PUBLIC, 'r') as public:
            self.assertTrue(verify(token, public.read()))

    def test_generate_from_encrypted_key_file(self):
        header = create_auth_header('foo', key_file=KEY2_PRIVATE, key_password=KEY2_PASSWORD)
        self.assertTrue(header.startswith('JWT '))
        token = header.split(' ', 1)[1]
        with open(KEY2_PUBLIC, 'r') as public:
            self.assertTrue(verify(token, public.read()))



#!/usr/bin/env python
from distutils.core import setup

packages = [
    'asymmetric_jwt_auth',
    'asymmetric_jwt_auth.test',
    'asymmetric_jwt_auth.management',
    'asymmetric_jwt_auth.management.commands',
    'asymmetric_jwt_auth.migrations',
]

requires = [
    'PyJWT>=1.4.0',
    'cryptography>=1.0',
]

setup(
    name='asymmetric_jwt_auth',
    version='0.1.6',
    description='Asymmetric key based authentication for HTTP APIs',
    long_description=open('README.rst').read(),
    author='Craig Weber',
    author_email='crgwbr@gmail.com',
    url='https://github.com/crgwbr/asymmetric_jwt_auth',
    packages=packages,
    install_requires=requires,
    license='LICENSE.md'
)



from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import (
    Encoding,
    PrivateFormat,
    PublicFormat,
    NoEncryption
)
import os.path
import asymmetric_jwt_auth.token as token


AUTH_METHOD = 'JWT'
ENCODING = 'utf-8'


def generate_key_pair(size=2048, public_exponent=65537, as_string=True):
    private = rsa.generate_private_key(
        public_exponent=public_exponent,
        key_size=size,
        backend=default_backend()
    )
    public = private.public_key()

    if not as_string:
        return private, public

    pem_private = private.private_bytes(Encoding.PEM, PrivateFormat.PKCS8, NoEncryption()).decode(ENCODING)
    pem_public = public.public_bytes(Encoding.PEM, PublicFormat.SubjectPublicKeyInfo).decode(ENCODING)
    return pem_private, pem_public


def load_private_key(key_file, key_password=None):
    key_file = os.path.expanduser(key_file)
    key_file = os.path.abspath(key_file)

    if not key_password:
        with open(key_file, 'r') as key:
            return key.read()

    with open(key_file, 'rb') as key:
        key_bytes = key.read()
    return decrypt_key(key_bytes, key_password).decode(ENCODING)


def decrypt_key(key, password):
    private = serialization.load_pem_private_key(key, password=password, backend=default_backend())
    return private.private_bytes(Encoding.PEM, PrivateFormat.PKCS8, NoEncryption())


def create_auth_header(username, key=None, key_file="~/.ssh/id_rsa", key_password=None):
    """Create an HTTP Authorization header using a private key file

    username - The username to authenticate as on the remote system
    key - Optional. A private key as either a string or an instance of
          cryptography.hazmat.primitives.asymmetric.rsa.RSAPrivateKey
    key_file - Path to a file containing the user's private key. Defaults
               to ~/.ssh/id_rsa. Should be in PEM format.
    key_password - Password to decrypt key_file. Should be a bytes object
    """
    if not key:
        key = load_private_key(key_file, key_password)
    claim = token.sign(username, key)
    return "%s %s" % (AUTH_METHOD, claim.decode(ENCODING))
